#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from operator import mod
import os
from tdf_tools.python_gitlab_api import GitlabAPI
from tdf_tools.tdf_print import printStage, printError, printTitle, printStr
import subprocess

from tdf_tools.dir_fixed import goInTdfFlutterDir


class GitlabUtils(object):

    def __init__(self):
        printStage("gitlab管理")

    def _runCmd(cmd):
        subprocess_result = subprocess.Popen(
            cmd, shell=True, stdout=subprocess.PIPE)
        subprocess_return = subprocess_result.stdout.read()
        return subprocess_return.decode('utf-8')

    def diff(self, targetBranch='master'):
        res = GitlabUtils._runCmd(
            "git diff --name-only {0}..{1}".format(self._getCurBranch(), targetBranch)).splitlines()
        if len(res) > 0:
            printStr("{0} files diff".format(len(res)))

    def commit(self, message):
        printStr(os.popen("git add .").read())
        printStr(os.popen("git commit -m {0}".format(message)).read())

    def status(self):
        res = GitlabUtils._runCmd("git status -s")
        if len(res.splitlines()) > 0:
            printStr(res)
        else:
            printStr('无文件要提交，干净的工作区')

    def unChangeValidate(self):
        res = GitlabUtils._runCmd("git status -s")
        if len(res.splitlines()) > 0:
            print("本地仍有修改，请先提交代码")
            printStr(res)
        else:
            exit(1)

    def clone(self, module, gitUrl):
        # 这边给clone下的项目重命名一下，避免仓库名和模块名不一致
        res = os.popen('git clone {0} {1}'.format(
            gitUrl, module)).read()
        printStr(res)

    def merge(self, source_branch):
        cmd = 'git merge {0}'.format(source_branch)
        state = os.system(cmd)
        GitlabUtils.checkStateWithMsg(state, cmd)

    # 一个模块创建MR
    # def mergeRequestCreateForOneModule(self, module):
        # api = GitlabAPI()
        # printStr("模块{0}创建MR：from <{1}> into <{2}>".format(
        #     module, self.initJsonData['featureBranch'], self.initJsonData['testBranch']))
        # api.createMR(
        #     self.moduleJsonData[module]['id'], self.initJsonData['featureBranch'], self.initJsonData['testBranch'])

        # 为所有模块创建MR

    def mergeRequestCreate(self, module, sourceBranch, targetBranch):
        api = GitlabAPI()
        GitlabUtils.unChangeValidate()
        printStr("模块{0}创建MR：from <{1}> into <{2}>".format(
            module, sourceBranch, targetBranch))
        api.createMR(
            self.moduleJsonData[module]['id'], sourceBranch, targetBranch)
        # 命令形式提交mr
        # for module in moduleNameList:
        #     moduleDir = os.path.join(projectModuleDir, module)
        #     os.chdir(moduleDir)
        #     printTitle(module)

        #     # 确保本地代码都应提交了，以免出现代码有忘记提交的，导致提测代码不是最新的
        #     GitlabUtils.unChangeValidate()

        #     cmd = 'git push -o merge_request.create -o merge_request.source={0} -o merge_request.target={1} -o merge_request.assignee_id={2}'.format(
        #         initJsonData['featureBranch'], initJsonData['testBranch'], "819")
        #     state = os.system(cmd)
        #     GitlabUtils.checkStateWithMsg(state, cmd)
        # print()

    def _getCurBranch(self):
        return GitlabUtils._runCmd(
            'git rev-parse --abbrev-ref HEAD').replace("\n", "")

    def fetch(self):
        cmd = 'git fetch'
        state = os.system(cmd)
        GitlabUtils.checkStateWithMsg(state, cmd)

    def pull(self):
        cmd = 'git pull'
        state = os.system(cmd)
        GitlabUtils.checkStateWithMsg(state, cmd)

    def checkout(self, featureBranch):
        cmd = 'git checkout {0}'.format(featureBranch)
        state = os.system(cmd)
        GitlabUtils.checkStateWithMsg(state, cmd)

    def createAndCheckout(self, featureBranch, shouldPush=False):
        cmd = 'git checkout -b {0}'.format(featureBranch)
        state = os.system(cmd)
        GitlabUtils.checkStateWithMsg(state, cmd)

        if shouldPush:
            cmd = 'git push --set-upstream origin {0}'.format(featureBranch)
            state = os.system(cmd)
            GitlabUtils.checkStateWithMsg(state, cmd)

    # def _branchJudge(self):
    #     isSame = True
    #     curBranch = ''
    #     for module in self.moduleNameList:
    #         moduleDir = os.path.join(projectModuleDir, module)
    #         os.chdir(moduleDir)
    #         cmd = f'git branch'
    #         branchList = GitlabUtils._runCmd(cmd).splitlines()
    #         for branch in branchList:
    #             if branch.startswith('*'):
    #                 if curBranch == '':
    #                     curBranch = branch.split(' ')[1]
    #                 elif curBranch != branch.split(' ')[1]:
    #                     isSame = False
    #     return isSame

    def push(self):
        pushCmd = 'git push'
        state = os.system(pushCmd)
        GitlabUtils.checkStateWithMsg(state, pushCmd)

    def checkStateWithMsg(state, cmd):
        if state != 0:
            printError(f"execute '{cmd}' failed")

    def executeRawCommand(self, rawCmd):
        printStr(GitlabUtils._runCmd(rawCmd))
        # print(os.popen(rawCmd).read())



# for example:
# python3 tz_router.py    # 遍历所有子模块生成路由map并注册到壳内
# python3 tz_router.py 模块名   # 为模块（如：tdf_account_module）生成路由描述文件，并遍历所有子模块生成路由map并注册到壳内

# 上述命令执行完会自动更新壳应用的路由文档

import os
from hashlib import md5
import shutil
import sys
import platform
from tdf_tools.dir_fixed import goInShellDir, getShellDir

isWindow = platform.system().lower() == 'windows'

osTargetCommand = ''
if(isWindow):
    osTargetCommand = 'cd'
else:
    osTargetCommand = 'pwd'

isUpdateTargetModule = False
targetModule = ''

macDownDataList = []  # 文档数据
mainPageData = []  # 业务入口页列表

packagesList = []  # 包含TZRouter注解页面的模块lib目录列表 item => {libPath:"", libName: ""}

entryPagePath = 'tdf-flutter://2dfire.com/business/entry'
entryPageNote = 'flutter entry'
entryPageWidget = 'TZRouterBusinessEntryPage'


class PackageNode:
    def __init__(self):
        self.packageName = ''
        # 存储的路径都是相对于shellPath的路径，所以在进入前需要先执行os.chdir(shellPath)
        self.packagePath = ''
        self.isRemote = False


# 备份壳模块路径
shellPath = os.popen(osTargetCommand).read().split('\n')[0]

# 判断业务组件下是否有依赖必须的build_runner和tdf_router_anno两个模块


def judgeHasBuildRunnerPackage(dirPath):
    hasBuildRunner = False
    hasRouterAnno = False
    os.chdir(dirPath)
    if (os.path.exists('.packages')):
        with open(".packages") as lines:
            for line in lines:
                if (line.find("build_runner") != -1):
                    hasBuildRunner = True
                if (line.find("tdf_router_anno") != -1):
                    hasRouterAnno = True
    return hasBuildRunner and hasRouterAnno

# 源码依赖模块路径获取


def getSourceCodePackagePath(line):
    packageName = line.split(':')[0]
    packagePath = line.split(':')[1].replace("%25", "%")
    if (packageName.find('tdf') != -1):
        packageNode = PackageNode()
        packageNode.packageName = packageName
        packageNode.packagePath = packagePath[0:len(packagePath) - 5]
        packageNode.isRemote = False
        return packageNode

# 远程依赖的模块生成代码


def getRemoteCodePackagePath(line):
    packageName = line.split(':file://')[0]
    packagePath = line.split(':file://')[1].replace("%25", "%")
    if (packageName.find('tdf') != -1):
        packageNode = PackageNode()
        packageNode.packageName = packageName
        if (isWindow):
            packageNode.packagePath = packagePath[1:len(packagePath) - 5]
        else:
            packageNode.packagePath = packagePath[0:len(packagePath) - 5]
        packageNode.isRemote = True
        return packageNode

# 生成代码


def generatorFunc():
    os.system('flutter packages pub run build_runner clean')
    os.system(
        'flutter packages pub run build_runner build --delete-conflicting-outputs')

# 判断指定目录下是否存在.tdf_router.dart结尾的文件


def hasRouterDartFile(dirPath):
    os.chdir(dirPath)
    nowaPath = os.popen(osTargetCommand).read().split("\n")[0]
    for root, dirs, files in os.walk(nowaPath):
        for file in files:
            src_file = os.path.join(root, file)
            if src_file.endswith('.tdf_router.dart'):
                return True
    return False


def defaultImportCreator(mFileWriter, content):
    mFileWriter.write("import {0}\n".format(content))

# 生成import数据


def importDataCreator(mFileWriter, itemModuleName, mFilePath, itemFileAlias):
    mFileWriter.write(
        "import \"package:{0}/{1}\" as {2};\n".format(itemModuleName, mFilePath, itemFileAlias))

# 路由map类生成器


def groupRouterClassCreator(mFileWriter, moduleName, list):
    content = """
routerMap() {{
    Map<String, TDFPageBuilder> map = Map();
    {1}
    return map;
}}


"""
    mFileWriter.write(content.format(moduleName, mapAddListCreator(list)))


def mapAddListCreator(list):
    content = ""
    for item in list:
        content += """map.putIfAbsent({0}.getRouterPath(), () =>  (String pageName, Map params, String _) => {1}.getPageWidget(params));
    """.format(item, item)
    return content


def mapMainPageAddListCreator(list):
    content = ""
    for item in list:
        content += """mainPageMap.putIfAbsent({0}.getNote(), {1}.getRouterPath());
    """.format(item, item)
    return content


def mapListCombineCreater(list):
    content = ""
    for item in list:
        content += "map.addAll({0}.routerMap());\n".format(item)
    content += "map.putIfAbsent(\"{0}\", () => (String pageName, Map params, String _) => {1}(params));".format(
        entryPagePath, entryPageWidget)
    return content


class routerNode:
    def __init__(self):
        self.package = ''
        self.routerPath = ''
        self.filePath = ''
        self.note = ''
        self.pageWidget = ''
        self.isMainPage = False


def addInMacDownData(routerFileList):
    for fileItem in routerFileList:
        with open(fileItem) as lines:
            node = routerNode()
            for line in lines:
                if (line.find("//#package#//") != -1):
                    node.package = line.split(
                        "//#package#//")[1].split("\n")[0]
                if (line.find("//#routerPath#//") != -1):
                    node.routerPath = line.split(
                        "//#routerPath#//")[1].split("\n")[0]
                if (line.find("//#filePath#//") != -1):
                    node.filePath = line.split(
                        "//#filePath#//")[1].split("\n")[0]
                if (line.find("//#note#//") != -1):
                    node.note = line.split("//#note#//")[1].split("\n")[0]
                if (line.find("//#pageWidget#//") != -1):
                    node.pageWidget = line.split(
                        "//#pageWidget#//")[1].split("\n")[0]
                if (line.find("//#isMainPage#//") != -1):
                    node.isMainPage = line.split(
                        "//#isMainPage#//")[1].split("\n")[0] == 'true'
            macDownDataList.append(node)
            if (node.isMainPage == True):
                mainPageData.append(node)


def addInWindowDownData(routerFileList):

    bytesTemp = bytes("\n", encoding="utf8")
    for fileItem in routerFileList:
        result = fileItem
        if(len(fileItem) > 260):
            temp = "\\\\?\\"
            temp2 = fileItem
            result = temp+fileItem
        with open(result, 'rb') as lines:
            node = routerNode()
            for line in lines:
                if (line.find(bytes(" //#package#//", encoding="utf8")) != -1):
                    node.package = line.split(
                        bytes("//#package#//", encoding="utf8"))[1].split(bytesTemp)[0]
                if (line.find(bytes("//#routerPath#//", encoding="utf8")) != -1):
                    node.routerPath = line.split(
                        bytes("//#routerPath#//", encoding="utf8"))[1].split(bytesTemp)[0]
                if (line.find(bytes("//#filePath#//", encoding="utf8")) != -1):
                    node.filePath = line.split(
                        bytes("//#filePath#//", encoding="utf8"))[1].split(bytesTemp)[0]
                if (line.find(bytes("//#note#//", encoding="utf8")) != -1):
                    node.note = str(line.split(
                        bytes("//#note#//", encoding="utf8"))[1].split(bytesTemp)[0], encoding="utf8")
                if (line.find(bytes("//#pageWidget#//", encoding="utf8")) != -1):
                    node.pageWidget = line.split(
                        bytes("//#pageWidget#//", encoding="utf8"))[1].split(bytesTemp)[0]
                if (line.find(bytes("//#isMainPage#//", encoding="utf8")) != -1):
                    node.isMainPage = line.split(
                        bytes("//#isMainPage#//", encoding="utf8"))[1].split(bytesTemp)[0] == 'true'
            macDownDataList.append(node)
            if (node.isMainPage == True):
                mainPageData.append(node)


def generateRouter(targetModule):
    goInShellDir()
    # 遍历壳应用.packages下的所有package,将所有tdf开头的模块都添加入packagesList中
    with open(".packages") as lines:
        for line in lines:
            goInShellDir()
            # os.chdir(shellPath)
            if (line.startswith('# ') is False):
                if (line.find(':file') != -1):
                    remotePackage = getRemoteCodePackagePath(line)
                    if (remotePackage != None):
                        packagesList.append(remotePackage)
                else:
                    sourcePackage = getSourceCodePackagePath(line)
                    if (sourcePackage != None):
                        packagesList.append(sourcePackage)

    # 上面已经拿到了所有的tdf库与其本地路径与是否是远程依赖
    isFind = False
    for item in packagesList:
        if (item.packageName == targetModule):
            isFind = True
            if (item.isRemote == False):
                if (judgeHasBuildRunnerPackage(item.packagePath)):
                    goInShellDir()
                    os.chdir(item.packagePath)
                    generatorFunc()
                else:
                    print("请确保模块{0}已依赖build_runner和tdf_router_anno两个包，可参考其他模块进行依赖配置❌".format(
                        targetModule))
            else:
                print("检测到模块{0}当前是远程依赖，无法生成路由描述文件❌".format(targetModule))

    if isFind == False and isUpdateTargetModule:
        print("未检测到模块{0}❌".format(targetModule))
        sys.exit(2)

    goInShellDir()
    os.chdir('lib')
    # os.chdir(shellPath + '/lib')
    if (os.path.exists('tz_router')):
        shutil.rmtree(r'tz_router')
    os.mkdir('tz_router')
    os.chdir('tz_router')
    os.mkdir('group')

    for packageItem in packagesList:
        # 存在路由描述文件才进行遍历
        goInShellDir()
        # os.chdir(shellPath)
        if (hasRouterDartFile(packageItem.packagePath)):
            os.chdir(shellPath + "/lib/tz_router/group/")
            os.mkdir(packageItem.packageName)
            os.chdir(packageItem.packageName)
            f = open('router_map.dart', 'w+')
            defaultImportCreator(f, "\'package:tdf_router/tdf_router.dart\';")

            goInShellDir()
            # os.chdir(shellPath)
            os.chdir(packageItem.packagePath + "/lib/")

            nowaPath = os.popen(osTargetCommand).read().split("\n")[0]

            routerAliasFileList = []
            routerFileList = []
            for root, dirs, files in os.walk(nowaPath):
                for file in files:
                    src_file = os.path.join(root, file)
                    if src_file.endswith('.tdf_router.dart'):
                        itemModuleName = packageItem.packageName
                        if(isWindow):
                            temp = src_file.split("\lib\\")[1]
                            itemFilePath = temp.replace("\\", "/")
                        else:
                            itemFilePath = src_file.split("/lib/")[1]
                        itemFileAlias = "tdf_router_" + \
                            md5(itemFilePath.encode()).hexdigest()[8:-8]

                        importDataCreator(f, itemModuleName,
                                          itemFilePath, itemFileAlias)
                        routerAliasFileList.append(itemFileAlias)
                        routerFileList.append(src_file)
            if (len(routerAliasFileList) > 0):
                print("模块{:>30}  检测到  {:>3}  个路由".format(
                    packageItem.packageName, len(routerAliasFileList)))
                groupRouterClassCreator(
                    f, packageItem.packageName, routerAliasFileList)
                if(isWindow):
                    addInWindowDownData(routerFileList)
                else:
                    addInMacDownData(routerFileList)

            f.close()

    goInShellDir()
    os.chdir('lib/tz_router')
    # os.chdir(shellPath + "/lib/tz_router")

    initF = open('init.dart', 'w+')

    # 生成初始化init文件
    if (os.path.exists("group") == False):
        os.mkdir('group')
    os.chdir('group')
    nowaPath = os.popen(osTargetCommand).read().split("\n")[0]
    defaultImportCreator(initF, "\'package:tdf_router/tdf_router.dart\';")
    # 添加flutter业务入口页面import
    defaultImportCreator(initF, "\'entry/tz_router_entry_page.dart\';")
    mapList = []
    for root, dirs, files in os.walk(nowaPath):
        for file in files:
            src_file = os.path.join(root, file)
            if (src_file.find("init.dart") != -1):
                continue
            if(isWindow):
                tmpItemFilePath = src_file.split("tz_router")[1]
                itemFilePath = tmpItemFilePath.replace("\\", "/")
            else:
                itemFilePath = src_file.split("tz_router")[1]
            itemFileAlias = "router_map_" + \
                md5(itemFilePath.encode()).hexdigest()[8:-8]
            importContent = "import \".{0}\" as {1};\n".format(
                itemFilePath, itemFileAlias)
            initF.write(importContent)
            mapList.append(itemFileAlias)

    initF.write(
        """

    class TZRouterManager {{
        static combineMap() {{
            Map<String, TDFPageBuilder> map = Map();
            {0}
            return map;
        }}
    }}
    """.format(mapListCombineCreater(mapList)))

    initF.close()

    def mainPageDataCreator():
        content = ""
        for item in mainPageData:
            if (item.isMainPage):
                content += "\"{0}\":\"{1}\",\n".format(
                    item.note, item.routerPath)
        return content

    goInShellDir()
    os.chdir('lib/tz_router')
    # os.chdir(shellPath + "/lib/tz_router")
    if (os.path.exists("entry") == False):
        os.mkdir('entry')

    os.chdir('entry')

    entryPageF = open('tz_router_entry_page.dart', 'w+')

    defaultImportCreator(entryPageF, "\'package:flutter/material.dart\';")
    defaultImportCreator(entryPageF, "\'package:tdf_router/tdf_router.dart\';")
    defaultImportCreator(
        entryPageF, "\'package:tdf_router_anno/router/router_anno.dart\';")
    defaultImportCreator(
        entryPageF, "\'package:tdf_base_utils/tdf_base_utils.dart\';")
    defaultImportCreator(
        entryPageF, "\'package:tdf_widgets/tdf_common_btn/tdf_btn.dart\';")
    defaultImportCreator(
        entryPageF, "\'package:tdf_widgets/title_bar/tdf_title_bar.dart\';")
    defaultImportCreator(
        entryPageF, "\'package:tdf_widgets/background/tdf_native_background.dart\';")
    entryPageF.write("""

    @TZRouter(path: '{0}', note: '{1}')
    class {2} extends StatefulWidget {{
        final Map params;

        {2}(this.params);

        @override
        _{2}State createState() => _{2}State();
    }}

    class _{2}State extends State<{2}> {{
        final TextEditingController routerController = new TextEditingController();
        final TextEditingController controller = new TextEditingController();
        final Map<String, String> pageMap = {{
        {3}
        }};

        @override
        Widget build(BuildContext context) {{
            TDFScreen.init(context);
            return Scaffold(
                backgroundColor: Color(0xffffffff),
                appBar: TDFTitleBar(
                    titleContent: \"{4}\",
                    backType: TDFTitleBackWidgetType.DEFAULT_BACK,
                ),
                body: SingleChildScrollView(
                    child: Column(
                        children: _getButtonList(),
                    ),
                ),
            );
        }}

        List<Widget> _getButtonList() {{
            List<Widget> list = [];
            list.add(_routerJumpWidget());
            list.add(Padding(
            padding: EdgeInsets.only(top: 20.w),
            child: TDFButton(
                content: \"根据路由跳转\",
                bgColor: 0xff0088ff,
                contentColor: 0xffffffff,
                onClick: () {{
                FocusScope.of(context).requestFocus(FocusNode());
                TDFRouter.open(routerController.text);
                }},
            ),
            ));
            list.add(_searchWidget());

            List<Widget> wrapEntrylist = [];
            pageMap.forEach((key, value) {{
            if (key.contains(controller.text)) {{
                wrapEntrylist.add(GestureDetector(
                    onTap: () {{
                    FocusScope.of(context).requestFocus(FocusNode());
                    TDFRouter.open(value);
                    }},
                    child: ClipRRect(
                    borderRadius: BorderRadius.all(Radius.circular(3.w)),
                    child: Container(
                        color: Colors.blue,
                        padding: EdgeInsets.only(
                            left: 7.w, right: 7.w, top: 4.w, bottom: 4.w),
                        child: Text(
                        key,
                        style: TextStyle(fontSize: 13.w),
                        ),
                    ),
                    )));
            }}
            }});
            list.add(SizedBox(
            height: 20.w,
            ));
            list.add(Wrap(
            spacing: 15.w,
            runSpacing: 10.w,
            children: wrapEntrylist,
            ));
            return list;
        }}

        Widget _routerJumpWidget() {{
            return TextField(
            controller: routerController,
            autofocus: false,
            decoration: InputDecoration(hintText: "输入页面路由"),
            onChanged: (value) {{
                setState(() {{}});
                }},
            );
        }}

        Widget _searchWidget() {{
            return TextField(
                controller: controller,
                autofocus: false,
                decoration: InputDecoration(hintText: "筛选"),
                onChanged: (value) {{
                    setState(() {{}});
                }},
            );
        }}
        

    }}
    """.format(entryPagePath, entryPageNote, entryPageWidget, mainPageDataCreator(), entryPageNote))
    entryPageF.close()

    goInShellDir()
    # os.chdir(shellPath)
    if (os.path.exists("路由文档") == True):
        shutil.rmtree(r'路由文档')
    os.mkdir("路由文档")
    os.chdir("路由文档")

    f = open('document.md', 'w+')

    goInShellDir()
    # os.chdir(shellPath)
    curDir = getShellDir()
    for root, dirs, files in os.walk(curDir):
        for file in files:
            src_file = os.path.join(root, file)
            if (src_file.find("pubspec.yaml") != -1):
                if(isWindow):
                    with open("pubspec.yaml", "rb") as lines:
                        for line in lines:
                            if (line.find(bytes("name: ", encoding="utf8")) != -1):
                                f.write("## Flutter壳应用（{0}）路由表\n".format(line.split(
                                    bytes("name: ", encoding="utf8"))[1].split(bytes("\n", encoding="utf8"))[0]))
                            break
                else:
                    with open("pubspec.yaml") as lines:
                        for line in lines:
                            if (line.find("name: ") != -1):
                                f.write("## Flutter壳应用（{0}）路由表\n".format(
                                    line.split("name: ")[1].split("\n")[0]))
                            break

    f.write("<table><thead><tr><th>{0}</th><th style = \"min-width:150px !important\">{1}</th><th>{2}</th><th>{3}</th><th>{4}</th></tr></thead>".format(
        "组件名", "描述", "路由path", "类名", "文件路径"))
    f.write("<tbody>")

    currentPackage = ""
    for item in macDownDataList:
        if (currentPackage == item.package):
            f.write("<tr><th>{0}</th><th>{1}</th><th>{2}</th><th>{3}</th><th>{4}</th></tr>\n".format(
                "", item.note, item.routerPath, item.pageWidget, item.filePath))
        else:
            currentPackage = item.package
            f.write("<tr><th>{0}</th><th>{1}</th><th>{2}</th><th>{3}</th><th>{4}</th></tr>\n".format(
                item.package, item.note, item.routerPath, item.pageWidget, item.filePath))

    f.write("</tbody></table>")

    f.close()



#!/usr/bin/env python
# encoding=utf-8

import json
import os
import subprocess
from tdf_tools.dir_fixed import goTdfCacheDir, goInShellDir
from tdf_tools.python_gitlab_api import GitlabAPI
from tdf_tools.tdf_print import printDebug, printStage, printStr
from ruamel import yaml

from tdf_tools.tdf_print import printError
from tdf_tools.dir_fixed import goInTdfFlutterDir


def _runCmd(cmd):
    subprocess_result = subprocess.Popen(
        cmd, shell=True, stdout=subprocess.PIPE)
    subprocess_return = subprocess_result.stdout.read()
    return subprocess_return.decode('utf-8')


def getShellBranch():
    featureBranch = _runCmd(
        'git rev-parse --abbrev-ref HEAD').replace("\n", "")
    return featureBranch


def shellModuleSet():
    appList = []
    idList = []
    for item in moduleJson:
        if moduleJson[item]['type'] == 'app':
            appList.append(item)
    printStr("壳模块列表：")
    i = 0
    for item in appList:
        i = i + 1
        idList.append(str(i))
        printStr("{0}.{1}".format(i, item))

    global selectedId
    while True:
        selectedId = input('请从以上模块中选择一个壳模块，输入序号即可：')
        if selectedId in idList:
            break

    i = 0
    for item in appList:
        i = i + 1
        if i == int(selectedId):
            print('选择模块：{0}'.format(item))
            return item


def createCache(moduleJson):
    global featureBranch
    global shellModule
    global moduleList
    # 初始化开发分支
    featureBranch = getShellBranch()
    # 初始化壳模块
    goInShellDir()

    with open('pubspec.yaml', 'r', encoding='utf-8') as rF:
        dic = yaml.round_trip_load(rF.read())
        if dic != None and dic.__contains__("name"):
            shellModule = dic['name']
        else:
            printError('读取壳模块模块名失败，请确保壳模块的pubspec.yaml配置了name属性')

    # 初始化开发的模块列表
    moduleList = []
    while True:
        moduleName = input('输入需要开发的模块名:(添加:+name，删除:-name，完成输入:!):')
        if moduleName != '':
            if moduleName == '!' or moduleName == '！':
                break
            if moduleJson.get(moduleName[1:], -1) != -1:
                if moduleName[0] == '+':
                    moduleList.append(moduleName[1:])
                    print('模块{0}已添加'.format(moduleName[1:]))
                elif moduleName[0] == '-':
                    moduleList.remove(moduleName[1:])
                    print('模块{0}已移除'.format(moduleName[1:]))
            else:
                if moduleName[0] == '+':
                    print("未从仓库中找到模块{0}，添加失败".format(moduleName[1:]))
                elif moduleName[0] == '-':
                    print("已添加模块列表中未找到模块{0}".format(moduleName[1:]))
            moduleList = list(dict.fromkeys(moduleList))

            print('已选模块列表：{0}'.format(moduleList))

    goTdfCacheDir()

    with open('initial_config.json', 'w', encoding='utf-8') as wf:
        initialDic = dict()
        initialDic['featureBranch'] = featureBranch
        initialDic['shellName'] = shellModule
        initialDic['moduleNameList'] = moduleList
        wf.write(json.dumps(initialDic, indent=2))
        wf.close()


def initModuleConfigJson():
    gitlabApi = GitlabAPI()
    moduleJson = gitlabApi.getModuleConfigJson()

    goTdfCacheDir()
    with open('module_config.json', 'w', encoding='utf-8') as wF:
        wF.write(json.dumps(moduleJson, indent=2))
        wF.close()
    return moduleJson


def getInitJsonData():  # 获取项目初始化数据
    goTdfCacheDir()
    with open('initial_config.json', 'r', encoding='utf-8') as readF:
        fileData = readF.read()
        readF.close()
        return json.loads(fileData)


def initialJsonFunc():
    global moduleJson

    goTdfCacheDir()

    if os.path.exists('module_config.json') == False:
        printStr('未发现module_config.json文件')
        printStr('创建module_config.json文件...')
        moduleJson = initModuleConfigJson()
        printStr('创建完成')
    else:
        with open('module_config.json', 'r', encoding='utf-8') as readF:
            moduleJson = json.loads(readF.read())
            readF.close()

    goTdfCacheDir()

    if os.path.exists('initial_config.json') == False:
        createCache(moduleJson)


def getModuleJsonData():  # 获取模块 git相关配置信息
    goTdfCacheDir()
    with open('module_config.json', 'r', encoding='utf-8') as readF:
        fileData = readF.read()
        readF.close()
        return json.loads(fileData)


def getModuleNameList():
    initJsonData = getInitJsonData()
    if (initJsonData.__contains__('moduleNameList') and isinstance(initJsonData['moduleNameList'], list)):
        moduleNameList = initJsonData['moduleNameList']
        return moduleNameList
    else:
        printError("❌ 请配置moduleNameList的值,以数组形式")



# 如果本地没有googletrans，请使用3.1.0a0这个版本，以下为安装示例：

# 如果是python3
# sudo python3 -m pip install googletrans==3.1.0a0

# 如果是python
# pip install googletrans==3.1.0a0

from email.policy import strict
import re
import os
import shutil
import json
from googletrans import Translator
import sys
import platform
from tdf_tools.dir_fixed import goInTdfFlutterDir, goInShellDir

isWindow = platform.system().lower() == 'windows'

osTargetCommand = ''
if(isWindow):
    osTargetCommand = 'cd'
else:
    osTargetCommand = 'pwd'

# 备份壳模块路径
# shellPath = os.popen(osTargetCommand).read().split('\n')[0]

# targetModule = ''

# 源码依赖模块路径获取


def getSourceCodePackagePath(line):
    if line.startswith("#") == False:
        packagePath = line.split(':')[1].replace("%25", "%")
        return packagePath[0:len(packagePath) - 5]


# refre = False
# localRefre = True
# sourcePackage = ''
# with open(".packages") as lines:
#     for line in lines:
#         goInShellDir()
#         # os.chdir(shellPath)
#         if line.find(':file') == -1:
#             if line.find(targetModule) != -1:
#                 refre = True
#                 sourcePackage = getSourceCodePackagePath(line)
#         if line.find(targetModule) != -1 and line.find(':file') != -1:
#             localRefre = False
#     if refre == False:
#         print("未引用模块{0}❌".format(targetModule))
#         sys.exit(2)
#     if localRefre == False:
#         print("本地引用的模块才能生成国际化代码❌")
#         sys.exit(2)

# if (sourcePackage != None or sourcePackage != ''):
#     os.chdir(sourcePackage)


# translator = Translator()


def getCurrentPath():
    return os.popen(osTargetCommand).read().split('\n')[0]


def tdf_translate(content, dest_lan):
    try:
        translator = Translator()
        text = translator.translate(content, src='zh-cn', dest=dest_lan).text
        return text
    except:
        print("{0} 翻译失败".format(content))
        return ""


genDir = getCurrentPath()


# def getCurrentModuleName():
#     os.chdir(genDir)
#     if(isWindow):
#         return getCurrentPath().split("\\")[len(getCurrentPath().split("\\")) - 1]
#     else:
#         return getCurrentPath().split("/")[len(getCurrentPath().split("/")) - 1]

# 获取目标文件内已经存在的所有字段名


def getTargetFileParamsList(targetFileName, mapName):
    with open(targetFileName, 'r', encoding='utf8') as readF:
        try:
            print("解析{0}文件内json数据".format(targetFileName))
            fileData = readF.read()
            fileJsonData = correctJsonData(
                fileData, mapName)
            jsonData = json.loads(fileJsonData, strict=False)
            jsonKeyList = []
            for item in jsonData:
                jsonKeyList.append(item)
            return jsonKeyList
        except Exception as e:
            print(e)
            exit(-1)


def correctJsonData(content, mapName):
    return content.replace("Map<String, String> {0} =".format(mapName), "")[:-1]


def getManagerClassName(moduleName):
    strList = moduleName.split("_")
    result = ""
    for item in strList:
        result = result + item.capitalize()
    return result


def mapListCreater(emptyParams):
    if emptyParams:
        return ""
    with open(tdfIntlDir + '/i18n.json', 'r', encoding='utf8') as jsonF:
        json_data = json.loads(jsonF.read())
        result = ""
        for keyItem in json_data:
            if(isWindow):
                result = result + \
                    "  /// -  {0}\n".format(
                        json_data[keyItem].replace("\n", ""))
            else:
                result = result + \
                    "  /// -  {0}\n".format(
                        json_data[keyItem].replace("\n", ""))
                result = result + \
                    "  static String get {0} => getKey(\"{1}\");\n".format(
                        keyItem, keyItem)
        jsonF.close()
        return result


# emptyParams参数是否为空
def generateManagerClass(emptyParams, moduleName):
    # 生成manager类
    os.chdir(tdfIntlDir)

    with open(moduleName + "_i18n.dart", "w+", encoding='utf8') as managerF:
        managerF.truncate()
        managerF.write("""
import 'package:{0}/tdf_intl/i18n/{0}_th.dart';
import 'package:{0}/tdf_intl/i18n/{0}_en.dart';
import 'package:{0}/tdf_intl/i18n/{0}_zh_ch.dart';
import 'package:{0}/tdf_intl/i18n/{0}_zh_tw.dart';
import 'package:tdf_base_utils/src/data/language_data.dart';

class {1}I18n {{

  static Map<String, Map<String, String>>? i18nMap;
  static Map<String, Map<String, String>> getInstance() {{
    if (i18nMap == null) {{
      i18nMap = Map();
      i18nMap!["zh-ch"] = zhchMap;
      i18nMap!["zh-tw"] = zhtwMap;
      i18nMap!["en"] = enMap;
      i18nMap!["th"] = thMap;
    }}
    return i18nMap!;
  }}
}}

extension {1}IntlStringExtension on String {{
    String get intl => {1}I18n.getInstance()[TDFLanguage.getLanguage()]?[this] ??
        {1}I18n.getInstance()[TDFLanguage.getDefaultLanguage()]
        ?[this] ??
        this;
}}

""".format(moduleName, getManagerClassName(moduleName)))


# 入口
def generateTranslate(targetModule):

    global tdfIntlDir
    tdfIntlDir = ""
    if (os.path.exists('lib')):

        os.chdir(genDir + '/lib')
        if (os.path.exists('tdf_intl') == False):
            os.mkdir('tdf_intl')

    # 进入tdf_intl目录
    # tdf_intl文件夹下包含i18n文件夹，其中存放4个dart文件，分别对应四种语言
    # tdf_intl文件夹下还包含一个origin.txt文件，用于存放需要转化的文案，每个文案为一行
    goInTdfFlutterDir()
    os.chdir(targetModule)
    os.chdir('lib')
    if os.path.exists('tdf_intl') == False:
        os.mkdir('tdf_intl')
    os.chdir('tdf_intl')
    tdfIntlDir = getCurrentPath()
    os.chdir(tdfIntlDir)
    # 如果没有i18n文件夹则生成
    if (os.path.exists('i18n') == False):
        os.mkdir('i18n')

    os.chdir(tdfIntlDir)

    i18nList = ['zh-ch', 'en', 'th', 'zh-tw']

    if (os.path.exists('i18n.json') == False):
        initF = open(tdfIntlDir + '/i18n.json', 'w+', encoding='utf8')
        initF.write("{}")
        initF.close

        for targetFileNameSuffix in i18nList:
            os.chdir(tdfIntlDir + '/i18n')
            targetFileName = targetModule + "_" + \
                targetFileNameSuffix.replace("-", "_") + ".dart"

            if os.path.exists(targetFileName) == False:
                newI18nFile = open(targetFileName, "w+")
                newI18nFile.write(
                    "Map<String, String> " + targetFileNameSuffix.replace("-", "") + "Map = {\n};")
                newI18nFile.close

            generateManagerClass(True, targetModule)
    else:
        with open(tdfIntlDir + '/i18n.json', 'r', encoding='utf8') as originF:
            json_data = json.loads(originF.read())

            # 遍历i18n目录下的四个存放语言的dart文件
            for targetFileNameSuffix in i18nList:
                os.chdir(tdfIntlDir + '/i18n')
                print(
                    "------------------------------------------------------------------------------------")
                targetFileName = targetModule + "_" + \
                    targetFileNameSuffix.replace("-", "_") + ".dart"

                paramsResultList = getTargetFileParamsList(
                    targetFileName, targetFileNameSuffix.replace("-", "") + "Map")

                os.chdir(tdfIntlDir + '/i18n')

                with open(targetFileName, 'r', encoding='utf8') as targetFile:
                    fileContent = targetFile.read()
                    keyNode = fileContent.find("}")

                    addStr = ""
                    for item in json_data:
                        if len(paramsResultList) > 0 and item in paramsResultList:
                            continue
                        else:
                            if keyNode != -1:
                                if (targetFileNameSuffix.find('zh-ch') != -1):
                                    # print(json_data[item])
                                    addStr = addStr + \
                                        "\n  \"{0}\": \"{1}\",\n".format(
                                            item, json_data[item])
                                elif (targetFileNameSuffix.find('zh-tw') != -1):
                                    print(json_data[item])
                                    translateText = tdf_translate(
                                        json_data[item], targetFileNameSuffix)
                                    if translateText != "":
                                        addStr = addStr + "\n  \"{0}\": \"{1}\",\n".format(
                                            item, translateText)
                                # 其他语言不再进行翻译，自行填写
                                # else:
                                    # addStr = addStr + "\n  \"{0}\": \"{1}\",\n".format(item, tdf_translate(json_data[item], targetFileNameSuffix))

                    newFileContent = fileContent[:keyNode] + \
                        addStr + fileContent[keyNode:]
                    targetInertFile = open(
                        targetFileName, 'w+', encoding='utf8')
                    targetInertFile.write(newFileContent)
                    targetInertFile.close
                    print("\n新增: \n{0}\n".format(addStr))
                    targetFile.close
            originF.close()
            generateManagerClass(False, targetModule)



# #!/usr/bin/env python
# # -*- coding: UTF-8 -*-
# # 模块重写脚本

# import os
# import json
# from enum import Enum
# from tdf_print import printError

# # 包含必要的数据校验
# # 校验规则：配置的模块列表中必须存在且只存在一个壳应用


# curDir = os.getcwd()

# projectGenPath = curDir  # 壳工程根目录
# initJsonFile = 'initial_config.json'  # 流程脚本初始化信息配置json文件
# moduleJsonFile = 'module_config.json'  # 流程脚本模块git相关信息json文件
# # 子module的存放目录，该目录会被 gitignore 忽略
# projectModuleDir = curDir + '/.tdf_flutter/.project_module'


# def goProjectDir():
#     os.chdir(curDir)
#     os.chdir('.tdf_flutter')


# # # 获取缓存文件中的initial_config.json数据
# # def getInitialJson():
# #     global initialJson

# def getInitJsonData():  # 获取项目初始化数据
#     goProjectDir()
#     with open(initJsonFile, 'r', encoding='utf-8') as readF:
#         fileData = readF.read()
#         readF.close()
#         return json.loads(fileData)


# def getModuleJsonData():  # 获取模块 git相关配置信息
#     goProjectDir()
#     with open(moduleJsonFile, 'r', encoding='utf-8') as readF:
#         fileData = readF.read()
#         readF.close()
#         return json.loads(fileData)


# def getModuleNameList():
#     initJsonData = getInitJsonData()
#     if (initJsonData.__contains__('moduleNameList') and isinstance(initJsonData['moduleNameList'], list)):
#         moduleNameList = initJsonData['moduleNameList']
#         moduleNameList.append(initJsonData['shellName'])
#         return moduleNameList
#     else:
#         printError("❌ 请配置moduleNameList的值,以数组形式")


# def moduleExistJudge():
#     moduleNameList = getModuleNameList()
#     os.chdir(projectModuleDir)
#     for item in moduleNameList:
#         if os.path.exists(item):
#             printError("init前请清空.project_module文件夹")


# def branchInvalidate():
#     initJsonData = getInitJsonData()
#     if (isExistKey(initJsonData, 'featureBranch') and initJsonData['featureBranch'] != '') == False:
#         printError("feature和test分支必须配置，详情请参考README.md")


# def moduleInvalidate():
#     initJsonData = getInitJsonData()
#     if initJsonData['shellName'] == "":
#         printError("模块校验不通过，请确保：shellName必须配置")


# def isExistKey(data, tagkey):
#     key_list = data.keys()
#     for key in key_list:
#         if(key == tagkey):
#             return True
#     return False



#!/usr/bin/env python
# encoding=utf-8

import os

from tdf_tools.tdf_print import printDebug, printError

genDirName = '.tdf_flutter'

# 向上遍历寻找.tdf_flutter存在的目录，找到则


def goSearchDir():
    os.chdir(os.path.abspath(r".."))
    curDir = os.getcwd()

    if os.path.exists(genDirName):
        return

    ex = Exception('未发现.tdf_flutter目录')
    raise ex


global curDir
curDir = os.getcwd()


def getShellDir():
    return curDir


def goInShellDir():
    os.chdir(curDir)
    # os.chdir(os.path.abspath(os.path.dirname(__file__)))
    # printDebug(os.getcwd())

# 进入.tdf_flutter文件夹


def goInTdfFlutterDir():
    goInShellDir()
    if os.path.exists('pubspec.yaml'):
        try:
            goSearchDir()
        except:
            goInShellDir()
            os.chdir(os.path.abspath(r".."))
            os.mkdir('.tdf_flutter')
    os.chdir('.tdf_flutter')


# 进入缓存文件目录


def goTdfCacheDir():
    goInShellDir()
    if os.path.exists('tdf_cache'):
        os.chdir('tdf_cache')
    elif os.path.exists('tdf_cache') is not True:
        if os.path.exists('pubspec.yaml') is not True:
            printError('请确保当前在flutter壳目录内')
        create = input('当前目录没有找到tdf_cache缓存文件夹，是否创建？(y/n):')
        if create == 'y':
            os.mkdir('tdf_cache')
        else:
            print('Oh,it\'s disappointing.')
            exit(1)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# 依赖分析【有向无环图，有向有环图】


import json
import os
from ruamel import yaml
import copy
from tdf_tools.tdf_print import printStage, printError, printStr
from tdf_tools.initial_data import getInitJsonData, getModuleJsonData, getModuleNameList
from tdf_tools.dir_fixed import goInShellDir, goInTdfFlutterDir


class DependencyNode:
    def __init__(self):
        self.nodeName = ''
        self.parent = []  # 父亲节点列表
        self.children = []  # 子孙节点列表
        self.isShell = False  # 是否是shell模块
        self.modulePath = ''  # 模块路径
        self.delete = False


class DependencyAnalysis(object):

    def __init__(self):
        printStage("依赖分析")
        self.initJsonData = getInitJsonData()
        self.moduleJsonData = getModuleJsonData()
        self.moduleNameList = getModuleNameList()
        self.shellName = self.initJsonData['shellName']

    # 分析lock文件，获取所有的packages
    def _analysisLock(self):
        # 读取lock内容
        with open('pubspec.lock', encoding='utf-8') as f:
            doc = yaml.round_trip_load(f)
            if (isinstance(doc, dict) and doc.__contains__('packages')):
                f.close()
                return doc['packages']

    # 生成依赖图
    def _generateDependenciesMap(self):
        for package in self.__moduleDependenciesMap:
            for module in self.moduleNameList:
                if package == module:
                    # 到这一步表明当前这个模块属于开发模块且在当前模块的依赖模块列表中，是当前模块的子模块
                    self._mNodeDict[self.__moduleName].children.append(package)
                    self._mNodeDict[package].parent.append(self.__moduleName)

    # 返回二维数组，用于并发打tag
    def _generateDependenciesOrder(self):
        resList = []
        while self._existNode():
            itemList = []

            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if not node.delete:
                        if len(node.children) == 0:
                            itemList.append(node.nodeName)
                            node.delete = True

            deleteNodeList = []
            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if node.delete:
                        deleteNodeList.append(node.nodeName)

            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    for deleteItem in deleteNodeList:
                        if node.children.__contains__(deleteItem):
                            node.children.remove(deleteItem)

            if len(itemList) == 0:
                break
            resList.append(itemList)
        return json.dumps(resList)

    # 返回一维数组，用于从下至上执行upgrade
    def _generateDependenciesOrderForUpgrade(self):
        resList = []
        while self._existNode():
            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if not node.delete:
                        if len(node.children) == 0:
                            resList.append(node.nodeName)
                            node.delete = True

            deleteNodeList = []
            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if node.delete:
                        deleteNodeList.append(node.nodeName)

            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    for deleteItem in deleteNodeList:
                        if node.children.__contains__(deleteItem):
                            node.children.remove(deleteItem)

        print(resList)

        return resList

    def _existNode(self):
        for item in self._mNodeDict:
            node = self._mNodeDict[item]
            if isinstance(node, DependencyNode) and node.delete == False:
                return True
        return False

    def _subChildCount(self, childName):
        for item in self._mNodeDict:
            node = self._mNodeDict[item]
            if isinstance(node, DependencyNode):
                if node.delete == False:
                    if node.children.__contains__(childName):
                        node.children.remove(childName)

    def generate(self):

        self._mNodeDict = dict()
        # 初始化子模块的节点列表
        for module in self.moduleNameList:
            node = DependencyNode()
            node.nodeName = module
            goInTdfFlutterDir()
            os.chdir(module)
            node.modulePath = os.getcwd()
            self._mNodeDict[module] = node
        # 初始化壳节点
        node = DependencyNode()
        node.nodeName = self.shellName
        node.isShell = True
        goInShellDir()
        node.modulePath = os.getcwd()
        self._mNodeDict[node.nodeName] = node

        # 读取子模块lock文件
        for module in self.moduleNameList:
            self.__moduleName = module
            # self.__moduleGenPath = projectModuleDir + "/" + module
            os.chdir(self._mNodeDict[module].modulePath)
            self.__moduleDependenciesMap = self._analysisLock()
            self._generateDependenciesMap()
        # 读取壳模块lock文件
        self.__moduleName = self.shellName
        os.chdir(self._mNodeDict[self.shellName].modulePath)
        self.__moduleDependenciesMap = self._analysisLock()
        self._generateDependenciesMap()

        return self._generateDependenciesOrder()

    def getDependencyOrder(self):

        self._mNodeDict = dict()
        # 初始化子模块的节点列表
        for module in self.moduleNameList:
            node = DependencyNode()
            node.nodeName = module
            goInTdfFlutterDir()
            os.chdir(module)
            node.modulePath = os.getcwd()
            self._mNodeDict[module] = node
        # 初始化壳节点
        node = DependencyNode()
        node.nodeName = self.shellName
        node.isShell = True
        goInShellDir()
        node.modulePath = os.getcwd()
        self._mNodeDict[node.nodeName] = node

        # 读取子模块lock文件
        for module in self.moduleNameList:
            self.__moduleName = module
            # self.__moduleGenPath = projectModuleDir + "/" + module
            os.chdir(self._mNodeDict[module].modulePath)
            self.__moduleDependenciesMap = self._analysisLock()
            self._generateDependenciesMap()
        # 读取壳模块lock文件
        self.__moduleName = self.shellName
        os.chdir(self._mNodeDict[self.shellName].modulePath)
        self.__moduleDependenciesMap = self._analysisLock()
        self._generateDependenciesMap()

        return self._generateDependenciesOrderForUpgrade()
